/*
 ******************************************
 **	   © by Widgetschmie.de 2005-2008	 **
 ******************************************
 */

// work around a bug in webkit (and while we're at it, use this to draw all canvasses when the shows are already displayed)
function redrawElapsed()
{
	for(var id in showHash)
	{
		showHash[id].drawCanvas();
	}
}



// draws a dash
function drawShowOver(context, is_active) {
	var line = canvasWidthHeight / 8;
	var x1 = canvasWidthHeight * 0.2;
	var x2 = canvasWidthHeight * 0.8;
	var y = canvasWidthHeight / 2;
	
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	context.beginPath();
	context.lineWidth = line;
	context.setStrokeColor(is_active ? "#333" : "#999");
	
	context.moveTo(x1,y);
	context.lineTo(x2,y);
	context.stroke();
}

// draws a piechart
function drawPiechart(context, timeelapsed, is_active) {
	var radius = 0.45 * canvasWidthHeight;
	var line = canvasWidthHeight / 12;
	
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	context.beginPath();
	context.arc(radius, radius, radius - (line / 2), 0, 6.2831853072, true);
	context.closePath();
	
	context.lineWidth = line;
	context.setStrokeColor(is_active ? "#333" : "#888");
	context.stroke();
	
	if (timeelapsed > 0)
	{
		context.beginPath();
		context.moveTo(radius, radius);
		// -1.5707963268 ^=  -90 degrees, 12 o'clock on a watch in this case
		context.arc(radius, radius, radius - (line / 2), -1.5707963268, (timeelapsed * 6.2831853072) - 1.5707963268, false);
		context.closePath();
		context.setFillColor(is_active ? "#333" : "#888");
		context.fill();
	}
}

// draws the alarm icon
function drawAlarmIcon(context, alarm_set, is_active) {
	var radius = canvasWidthHeight / 2;
	var line = canvasWidthHeight / 12;
	
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	context.lineWidth = line;
	context.setFillColor(alarm_set ? (is_active ? "#222" : "#444") : (is_active ? "rgb(60,60,60)" : "#AAA"));
	context.setStrokeColor(alarm_set ? (is_active ? "#222" : "#444") : (is_active ? "rgb(60,60,60)" : "#AAA"));
	
	// 45 degree in ARC: 0.785398163397		0.7854
	// 135 degree in ARC: 2.3561944902		2.3562
	
	context.beginPath();
	context.arc(radius, radius, 2 * line, 0, 2* Math.PI, false);
	context.fill();
	
	context.beginPath();
	context.arc(radius, radius, ((radius / 2) + (line / 2)), -0.7854, 0.7854, false);
	context.stroke();
	
	context.beginPath();
	context.arc(radius, radius, ((radius / 2) + (line / 2)), -2.3562, 2.3562, true);
	context.stroke();
	
	if(alarm_set) {
		context.beginPath();
		context.arc(radius, radius, (radius - (line / 2)), -0.7854, 0.7854, false);
		context.stroke();
		
		context.beginPath();
		context.arc(radius, radius, (radius - (line / 2)), -2.3562, 2.3562, true);
		context.stroke();
	}
}

// draws the cassette
function drawCassette(context, recording, is_active) {
	var width = 10 * canvasWidthHeight;			// rendering is much better when we apply the scaling below...
	var dist = 0.04 * width;
	var round = 0.04 * width;
	var height = 0.6 * width;
	
	context.save();
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	
	context.scale(0.1, 0.1);					// ...right here
	context.rotate(-0.174);
	context.translate(-0.08*width, 0.2*width);
	
	// cassette
	context.beginPath();
	context.setFillColor(recording ? (is_active ? "#700" : "#900") : (is_active ? "#333" : "#888"));
	context.moveTo(dist, (dist + 1*round));
	context.quadraticCurveTo(dist, dist, (dist + 1*round), dist);
	context.lineTo((width - dist - round), dist);
	context.quadraticCurveTo((width - dist), dist, (width - dist), (dist + 1*round));
	context.lineTo((width - dist), (dist + 1*height - round));
	context.quadraticCurveTo((width - dist), (dist + 1*height), (width - dist - round), (dist + 1*height));
	context.lineTo((dist + 1*round), (dist + 1*height));
	context.quadraticCurveTo(dist, (dist + 1*height), dist, (dist + 1*height - round));
	context.lineTo(dist, (dist + 1*round));
	context.fill();
	
	// color
	if(recording && (recording < 1)) {
		recording = parseFloat(recording);
		//alert(recording);
		context.beginPath();
		context.moveTo(dist, (dist + 1*round));
		context.quadraticCurveTo(dist, dist, (dist + 1*round), dist);
		context.lineTo(recording * (width - dist), dist);
		context.lineTo(recording * (width - dist), (dist + 1*height));
		context.lineTo((dist + 1*round), (dist + 1*height));
		context.quadraticCurveTo(dist, (dist + 1*height), dist, (dist + 1*height - round));
		context.lineTo(dist, (dist + 1*round));
		context.setFillColor(is_active ? "#700" : "#900");
		context.fill();
	}
	
	// 2 big holes
	context.beginPath();
	context.setFillColor(is_active ? "rgb(0,117,217)" : "#FFF");
	var rad = 0.21 * height;
	context.arc((dist + 1.75*rad), (dist + 1.75*rad), rad, 0, 2*Math.PI, true);
	context.arc((width - dist - 1.75*rad), (dist + 1.75*rad), rad, 0, 2*Math.PI, true);
	context.fill();
	
	// 4 small holes
	context.beginPath();
	rad = 0.1 * height;
	var w = (width - 2*dist) / 5;
	var y = (dist + 1*height) - 1.5*rad;
	context.arc((dist + 1*w), y, rad, 0, 2*Math.PI, true);
	context.arc((dist + 2*w), y, rad, 0, 2*Math.PI, true);
	context.arc((dist + 3*w), y, rad, 0, 2*Math.PI, true);
	context.arc((dist + 4*w), y, rad, 0, 2*Math.PI, true);
	context.fill();
	
	context.restore();
}

function drawEye(context, watching, is_active) {
	var width = canvasWidthHeight;
	var center = width / 2;
	var eyesize = 0.27 * width;
	
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	context.globalAlpha = watching ? 1.0 : 0.4;
	
	context.setStrokeColor("#000");
	context.setFillColor("#16D");
	context.lineWidth = width / 12;
	
	// eye
	context.beginPath();
	context.arc(center, center, eyesize, 0, 2*Math.PI, true);
	context.fill();
	
	context.beginPath();
	context.arc(center, center, eyesize, 0, 2*Math.PI, true);
	context.stroke();
	
	context.beginPath();
	context.setFillColor("#000");
	context.arc(center, center, 0.11*width, 0, 2*Math.PI, true);
	context.fill();
	
	context.beginPath();
	context.setFillColor("rgba(255,255,255,0.7)");
	context.arc((center - 0.06*width), (center - 0.06*width), 0.06*width, 0, 2*Math.PI, true);
	context.fill();
	
	// eyebrow
	context.beginPath();
	context.arc(center, width, 0.8*width, -0.7854, -2.3562, true);
	context.stroke();
}

function drawClosingEye(context, is_active) {
	var width = canvasWidthHeight;
	var center = width / 2;
	var eyesize = 0.27 * width;
	
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	context.setStrokeColor("#000");
	context.setFillColor("#16D");
	context.lineWidth = width / 12;
	
	context.beginPath();
	context.arc(center, 0, 0.8*width, 0.7854, 2.3562, false);
	context.stroke();
}

function drawiButton(context, is_active) {
	context.save();
	
	var width = canvasWidthHeight * 0.8;
	context.clearRect(0, 0, canvasWidthHeight, canvasWidthHeight);
	context.translate(width * 0.1, width * 0.1);		// center the i on the available canvas
	
	// circle around
	context.beginPath();
	context.arc(0.5*width, 0.5*width, 0.5*width, 0, 2*Math.PI, true);
	if(is_active) {
		context.setStrokeColor("rgba(0,0,0,0.7)");
		context.lineWidth = width / 14;
		context.stroke();
	}
	else {
		context.setFillColor("rgba(0,0,0,0.5)");
		context.fill();
	}
	
	// i-dot
	context.beginPath();
	context.setFillColor(is_active ? "rgba(0,0,0,0.7)" : "#FFF");
	context.arc(0.58*width, 0.18*width, 0.1*width, 0, 2*Math.PI, true);
	context.fill();
	
	// i-body
	context.beginPath();
	context.setFillColor(is_active ? "rgba(0,0,0,0.7)" : "#FFF");
	context.moveTo(0.6*width, 0.45*width);
	context.bezierCurveTo(0.63*width, 0.27*width, 0.4*width, 0.36*width, 0.35*width, 0.46*width);
	context.bezierCurveTo(0.3*width, 0.55*width, 0.33*width, 0.59*width, 0.36*width, 0.54*width);
	context.bezierCurveTo(0.4*width, 0.5*width, 0.46*width, 0.43*width, 0.44*width, 0.54*width);
	context.bezierCurveTo(0.43*width, 0.64*width, 0.41*width, 0.61*width, 0.38*width, 0.80*width);
	context.bezierCurveTo(0.35*width, width, 0.57*width, 0.88*width, 0.63*width, 0.79*width);
	context.bezierCurveTo(0.69*width, 0.7*width, 0.67*width, 0.65*width, 0.63*width, 0.7*width);
	context.bezierCurveTo(0.59*width, 0.75*width, 0.54*width, 0.81*width, 0.55*width, 0.71*width);
	context.bezierCurveTo(0.56*width, 0.62*width, 0.58*width, 0.63*width, 0.6*width, 0.45*width);
	context.closePath();
	context.fill();
	
	context.restore();
}

// draws either a (+), a (-) or a (x) button
function drawControlButton(context, type, is_active) {
	var width = canvasWidthHeight;
	var radius = ('closeButton' == type) ? 0.5*width : 0.4*width;
	var edge_dist = 0.25 * width;
	
	context.clearRect(0, 0, width, width);
	
	// circle
	context.beginPath();
	context.setFillColor(is_active ? "rgba(255,255,255,0.4)" : "rgba(255,255,255,0.6)");
	context.arc(0.5*width, 0.5*width, radius, 0, 2*Math.PI, true);
	context.fill();
	
	// the control
	context.beginPath();
	context.lineWidth = width / 8;
	context.setStrokeColor("#000");
	
	// draw an X
	if('closeButton' == type) {
		context.moveTo(edge_dist, edge_dist);
		context.lineTo((width - edge_dist), (width - edge_dist));	// the "\"
		context.moveTo(edge_dist, (width - edge_dist));
		context.lineTo((width - edge_dist), edge_dist);				// the "/"
	}
	
	// draw a - or a +
	else {
		context.moveTo(edge_dist, (width / 2));
		context.lineTo((width - edge_dist), (width / 2));			/// the "-"
		
		if('plusButton' == type) {
			context.moveTo((width / 2), edge_dist);
			context.lineTo((width / 2), (width - edge_dist));		/// the "|"
		}
	}
	
	context.stroke();
}


function drawStar(context, x_offset, is_active)
{
	var width = canvasWidthHeight;
	var dia = width / 2;
	
	context.clearRect(x_offset * width, 0, width, width);
	context.beginPath();
	
	for (i = 0; i <= 10; i++)
	{
		var x = dia + (Math.sin(.6282 * i) * (i % 2 == 1 ? dia : (dia * .45)));
		var y = dia + (Math.cos(.6282 * i) * (i % 2 == 1 ? dia : (dia * .45)));
		if (i == 0) {
			context.moveTo((x_offset * width) + x, y);
		}
		else {
			context.lineTo((x_offset * width) + x, y);
		}
	}
	
	context.closePath();
	
	// Pascal didn't like this:
	//var gradient = context.createRadialGradient((x_offset * width) + width*.5, width * 1.1, dia / 2, (x_offset * width) + width * .5, width * 2, width * 2);
	//gradient.addColorStop(0, "rgb(240,180,0)");
	//gradient.addColorStop(0.5, "rgb(200,0,0)");
	//gradient.addColorStop(1, "#FFF");
	//context.fillStyle = gradient;
	context.setFillColor(is_active ? "rgb(200,0,0)" : "rgba(0,0,0,0.25)");
	context.fill();		
}



// currently only implemented for the controlButtons (+) (-) and (x)
function canvasPressed(canv) {
	if(canv) {
		var context = canv.getContext("2d");
		var type = canv.getAttribute("lang");		// mis-use the lang-tag to store the type of the control button
		drawControlButton(context, type, true);
	}
}

function canvasReleased(canv) {
	if(canv) {
		var context = canv.getContext("2d");
		var type = canv.getAttribute("lang");
		drawControlButton(context, type, false);
	}
}

